@@ -1,8 +1,12 @@
 """Helpers for vcspull."""
 import os
+import pathlib
+from typing import Literal
+
+from vcspull._internal.config_reader import ConfigReader
 
 
-class EnvironmentVarGuard(object):
+class EnvironmentVarGuard:
 
     """Class to help protect the environment variable properly.
 
@@ -36,3 +40,12 @@ class EnvironmentVarGuard(object):
             self._environ[envvar] = value
         for unset in self._unset:
             del self._environ[unset]
+
+
+def write_config(config_path: pathlib.Path, content: str) -> pathlib.Path:
+    config_path.write_text(content, encoding="utf-8")
+    return config_path
+
+
+def load_raw(data: str, format: Literal["yaml", "json"]) -> dict:
+    return ConfigReader._load(format=format, content=data)
